package de.vernideas.mc.common.gen;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

import net.minecraft.block.Block;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.Event.Result;
import net.minecraftforge.event.terraingen.BiomeEvent;

public class BiomeSpecificBlock
{
	public final int blockID;
	public final int metadata;
	public final Block block;
	
	public final static BiomeSpecificBlock air = new BiomeSpecificBlock(0, 0);
	
	private final static Map<Pair<Block, Integer>, Pair<Block, Integer>> desertReplaceMap;
	
	private static BiomeSpecificBlock getDesertReplacement(Block original, Integer originalMeta)
	{
		Pair<Block,Integer> replacement = desertReplaceMap.get(ImmutablePair.<Block,Integer>of(original, originalMeta));
		if( null == replacement )
		{
			// Try the wildcard (-1)
			replacement = desertReplaceMap.get(ImmutablePair.<Block,Integer>of(original, -1));
		}
		// Found anything?
		if( null != replacement )
		{
			return new BiomeSpecificBlock(replacement.getLeft(), replacement.getRight() >= 0 ? replacement.getRight() : originalMeta);
		}
		else
		{
			return new BiomeSpecificBlock(original, originalMeta);
		}
	}
	
	/**
	 * Return vanilla replacements only
	 */
	public static BiomeSpecificBlock queryVanilla(Block block, int meta, BiomeGenBase biome)
	{
    	if( biome == BiomeGenBase.desert || biome == BiomeGenBase.desertHills )
    	{
    		return getDesertReplacement(block, meta);
    	}
    	else
    	{
    		return new BiomeSpecificBlock(block, meta);
    	}
	}
	
	public static BiomeSpecificBlock query(Block block, int meta, BiomeGenBase biome)
	{
		int newID = -1;
		int newMeta = -1;
		// Determine modded block ID
        BiomeEvent.GetVillageBlockID eventBlock = new BiomeEvent.GetVillageBlockID(biome, block.blockID, meta);
        MinecraftForge.TERRAIN_GEN_BUS.post(eventBlock);
        if( eventBlock.getResult() == Result.DENY )
        {
        	newID = eventBlock.replacement;
        }
		// Determine modded block metadata
        BiomeEvent.GetVillageBlockMeta eventMeta = new BiomeEvent.GetVillageBlockMeta(biome, block.blockID, meta);
        MinecraftForge.TERRAIN_GEN_BUS.post(eventMeta);
        if( eventMeta.getResult() == Result.DENY )
        {
        	newMeta = eventMeta.replacement;
        }
        
        if( newID >= 0 && newMeta >= 0 )
        {
        	// Mod-based replacement
            return new BiomeSpecificBlock(newID, newMeta);
        }
        else if( newID < 0 && newMeta < 0 )
        {
        	// Default replacement
        	if( biome == BiomeGenBase.desert || biome == BiomeGenBase.desertHills )
        	{
        		return getDesertReplacement(block, meta);
        	}
        	else
        	{
        		return new BiomeSpecificBlock(block, meta);
        	}
        }
        else
        {
        	// Hard cases - events replaced one value but not the other
        	if( newID >= 0 )
        	{
        		// Replaced the ID, keep the meta
        		if( biome == BiomeGenBase.desert || biome == BiomeGenBase.desertHills )
        		{
        			BiomeSpecificBlock desertReplacement = getDesertReplacement(block, meta);
        			return new BiomeSpecificBlock(newID, desertReplacement.metadata);
        		}
        		else
        		{
        			return new BiomeSpecificBlock(newID, meta);
        		}
        	}
        	else
        	{
        		// Replaced the meta, kept the ID
        		if( biome == BiomeGenBase.desert || biome == BiomeGenBase.desertHills )
        		{
        			BiomeSpecificBlock desertReplacement = getDesertReplacement(block, meta);
        			return new BiomeSpecificBlock(desertReplacement.block, newMeta);
        		}
        		else
        		{
        			return new BiomeSpecificBlock(block, newMeta);
        		}
        	}
        }
	}
	
	protected BiomeSpecificBlock(Block blockInstance, int meta)
	{
		blockID = blockInstance.blockID;
		metadata = meta;
		block = blockInstance;
	}
	
	protected BiomeSpecificBlock(int id, int meta)
	{
		blockID = id;
		metadata = meta;
		block = Block.blocksList[id];
	}
	
	private static void addDesertReplacement(Block from, Integer fromMeta, Block to, Integer toMeta)
	{
		desertReplaceMap.put(
				ImmutablePair.<Block, Integer>of(from, fromMeta),
				ImmutablePair.<Block, Integer>of(to, toMeta));
	}
	static
	{
		desertReplaceMap = new HashMap<Pair<Block,Integer>, Pair<Block,Integer>>();
		
		// Fill in default Minecraft replacements.
		addDesertReplacement(Block.wood, -1, Block.sandStone, 0);
		addDesertReplacement(Block.planks, -1, Block.sandStone, 0);
		addDesertReplacement(Block.cobblestone, -1, Block.sandStone, 0);
		addDesertReplacement(Block.cobblestoneMossy, -1, Block.sandStone, 1);
		addDesertReplacement(Block.gravel, -1, Block.sandStone, 0);
		addDesertReplacement(Block.stairsWoodOak, -1, Block.stairsSandStone, -1);
		addDesertReplacement(Block.stairsWoodJungle, -1, Block.stairsSandStone, -1);
		addDesertReplacement(Block.stairsCobblestone, -1, Block.stairsSandStone, -1);
		addDesertReplacement(Block.stairsWoodBirch, -1, Block.stairsNetherQuartz, -1);
		addDesertReplacement(Block.stairsWoodSpruce, -1, Block.stairsStoneBrick, -1);
		addDesertReplacement(Block.stairsWoodSpruce, -1, Block.stairsStoneBrick, -1);
		// Stone and cobblestone slabs
		addDesertReplacement(Block.stoneSingleSlab, 0, Block.stoneSingleSlab, 1);
		addDesertReplacement(Block.stoneSingleSlab, 3, Block.stoneSingleSlab, 1);
		addDesertReplacement(Block.stoneSingleSlab, 8, Block.stoneSingleSlab, 9);
		addDesertReplacement(Block.stoneSingleSlab, 11, Block.stoneSingleSlab, 9);
		addDesertReplacement(Block.stoneDoubleSlab, 0, Block.stoneDoubleSlab, 1);
		addDesertReplacement(Block.stoneDoubleSlab, 3, Block.stoneDoubleSlab, 1);
		// Wooden slabs
		for( int meta = 0; meta <= 5; ++ meta )
		{
			addDesertReplacement(Block.woodSingleSlab, meta, Block.stoneSingleSlab, 1);
			addDesertReplacement(Block.woodSingleSlab, meta + 8, Block.stoneSingleSlab, 9);
			addDesertReplacement(Block.woodDoubleSlab, meta, Block.stoneDoubleSlab, 1);
		}
		// Dirt to sand
		addDesertReplacement(Block.dirt, -1, Block.sand, 9);
	}
}
