package de.vernideas.mc.taverns.gen;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

import biomesoplenty.api.Blocks;
import biomesoplenty.api.BlockReferences.EnumBlocks;
import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.gen.structure.ComponentVillage;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraftforge.common.ForgeDirection;
import de.vernideas.mc.common.gen.BiomeSpecificBlock;
import de.vernideas.mc.taverns.Taverns;
import de.vernideas.mc.taverns.direction.Dir;

public abstract class ComponentVillageBase extends ComponentVillage
{
	public ComponentVillageBase() {}
	
	protected ComponentVillageBase(ComponentVillageStartPiece startPiece, int orient)
	{
		super(startPiece, orient);
	}

	// Helper methods for less writing and preparation for 1.7 changes
	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock, boolean alwaysreplace)
	{
		fillWithBlocks(worldObj, structBB,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, alwaysreplace);
	}

	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			BiomeSpecificBlock placeBlock, BiomeSpecificBlock replaceBlock, boolean alwaysreplace)
	{
		fillWithMetadataBlocks(worldObj, structBB,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, placeBlock.metadata, replaceBlock.blockID, replaceBlock.metadata, alwaysreplace);
	}

	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock)
	{
		fillWithBlocks(worldObj, structBB,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, false);
	}
	
	protected void fill(World worldObj, StructureBoundingBox structBB,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			BiomeSpecificBlock placeBlock, BiomeSpecificBlock replaceBlock)
	{
        for( int x = minX; x <= maxX; ++ x )
        {
            for( int y = minY; y <= maxY; ++ y )
            {
                for( int z = minZ; z <= maxZ; ++ z )
                {
            		if( x != minX && x != maxX && y != minY && y != maxY && z != minZ && z != maxZ )
            		{
            			// Not on the sides
            			placeBlock(worldObj, replaceBlock, x, y, z, structBB);
            		}
            		else
            		{
            			placeBlock(worldObj, placeBlock, x, y, z, structBB);
            		}
            	}
            }
        }
	}

	protected void fillRandomly(World world, StructureBoundingBox structBB, Random rnd, float chance,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			Block placeBlock, Block replaceBlock)
	{
		randomlyFillWithBlocks(world, structBB, rnd, chance,
				minX, minY, minZ, maxX, maxY, maxZ,
				placeBlock.blockID, replaceBlock.blockID, false);
	}
	
	protected void fillRandomly(World worldObj, StructureBoundingBox structBB, Random rnd, float chance,
			int minX, int minY, int minZ, int maxX, int maxY, int maxZ,
			BiomeSpecificBlock placeBlock, BiomeSpecificBlock replaceBlock)
	{
        for( int x = minX; x <= maxX; ++ x )
        {
            for( int y = minY; y <= maxY; ++ y )
            {
                for( int z = minZ; z <= maxZ; ++ z )
                {
                	if( rnd.nextFloat() <= chance )
                	{
                		if( x != minX && x != maxX && y != minY && y != maxY && z != minZ && z != maxZ )
                		{
                			// Not on the sides
                			placeBlock(worldObj, replaceBlock, x, y, z, structBB);
                		}
                		else
                		{
                			placeBlock(worldObj, placeBlock, x, y, z, structBB);
                		}
                	}
                }
            }
        }
	}

	protected void fillDownwards(World world,
			Block block, int par3, int xx, int par5, int zz,
			StructureBoundingBox structBB)
	{
		fillCurrentPositionBlocksDownwards(world, block.blockID, par3, xx, par5, zz, structBB);
	}

	protected void placeBlock(World world, Block block,
			int metadata, int posX, int posY, int posZ,
			StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, block.blockID, metadata, posX, posY, posZ, structBB);
	}
	
	protected void placeBlock(World world, Block block, ForgeDirection dir,
			int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, block.blockID, Dir.getMeta(block, dir, coordBaseMode), posX, posY, posZ, structBB);
	}
	
	protected void placeBlock(World world, BiomeSpecificBlock block,
			int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
        int globalX = this.getXWithOffset(posX, posZ);
        int globalY = this.getYWithOffset(posY);
        int globalZ = this.getZWithOffset(posX, posZ);

        if( structBB.isVecInside(globalX, globalY, globalZ) )
        {
        	world.setBlock(globalX, globalY, globalZ, block.blockID, block.metadata, 2);
        }
	}
	
	protected void placeAir(World world, int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		placeBlockAtCurrentPosition(world, 0, 0, posX, posY, posZ, structBB);
	}

	protected void placeDoor(World world, StructureBoundingBox structBB,
			Random rnd, int posX, int posY, int posZ, ForgeDirection dir)
	{
		placeDoorAtCurrentPosition(world, structBB, rnd, posX, posY, posZ, Dir.getMeta(Block.doorWood, dir, coordBaseMode));
	}
	
	/**
	 * Warning: Sets the position *below* this one to air!
	 */
	protected void placeTorch(World world, StructureBoundingBox structBB,
			int posX, int posY, int posZ, ForgeDirection dir)
	{
        placeBlock(world, Block.stone, 0, posX, posY - 1, posZ, structBB);
        placeBlock(world, Block.torchWood, dir, posX, posY, posZ, structBB);
        placeAir(world, posX, posY - 1, posZ, structBB);
	}
	
	protected void setMetadata(World world, int metadata,
			int posX, int posY, int posZ, StructureBoundingBox structBB)
	{
		int globalX = getXWithOffset(posX, posZ);
		int globalY = getYWithOffset(posY);
		int globalZ = getZWithOffset(posX, posZ);
		
		if( structBB.isVecInside(globalX, globalY, globalZ) )
		{
			world.setBlockMetadataWithNotify(globalX, globalY, globalZ, metadata, 2);
		}
	}
	
	/**
	 * Does nothing if amount <= 0.
	 * Else tries to spawn entities by calling getEntity(index) as often as needed.
	 * 
	 * @return Amount of entities spawned
	 */
    protected int spawnEntity(World world, StructureBoundingBox structBB, int posX, int posY, int posZ, int amount)
    {
    	int spawned = 0;
    	for( int idx = 0; idx < amount; ++ idx )
    	{
    		int globalX = getXWithOffset(posX, posZ);
    		int globalY = getYWithOffset(posY);
    		int globalZ = getZWithOffset(posX, posZ);
    		
    		if( structBB.isVecInside(globalX, globalY, globalZ) )
    		{
    			EntityLivingBase entity = getEntity(world, idx);
    			if( null != entity )
    			{
    				++ spawned;
    				entity.setLocationAndAngles((double)globalX + 0.5d, (double)globalY, (double)globalZ + 0.5d, 0.5f, 0.0f);
    				world.spawnEntityInWorld(entity);
    			}
    		}
    	}
    	
    	return spawned;
    }

    /**
     * Spawn a single villager. Return that villager or <i>null</i> if spawning failed, for further modification.
     */
    protected EntityVillager spawnVillager(World world, StructureBoundingBox structBB, int posX, int posY, int posZ)
    {
    	return spawnVillager(world, structBB, posX, posY, posZ, 0);
    }

    /**
     * Spawn a single villager. Return that villager or <i>null</i> if spawning failed, for further modification.
     */
    protected EntityVillager spawnVillager(World world, StructureBoundingBox structBB, int posX, int posY, int posZ, int index)
    {
		int globalX = getXWithOffset(posX, posZ);
		int globalY = getYWithOffset(posY);
		int globalZ = getZWithOffset(posX, posZ);

		EntityVillager entityvillager = null;
		if( structBB.isVecInside(globalX, globalY, globalZ) )
		{
			entityvillager = new EntityVillager(world, getVillagerType(index));
            entityvillager.setLocationAndAngles((double)globalX + 0.5D, (double)globalY, (double)globalZ + 0.5D, 0.0F, 0.0F);
            world.spawnEntityInWorld(entityvillager);
        }
		
		return entityvillager;
    }

    /**
     * Override this method for custom (non-villager) entity spawning.
     * 
     * @param world the World to spawn the entity in
     * @param index the index number of the entity
     * @return the EntityLivingBase to spawn, or null if we don't spawn any
     */
    protected EntityLivingBase getEntity(World world, int index)
    {
    	return null;
    }
    
    private Map<Integer, Block> biomesOPlentyStairMapping = null;
    private void initBOPStairMapping()
    {
    	if( null == biomesOPlentyStairMapping )
    	{
    		biomesOPlentyStairMapping = new HashMap<Integer, Block>(14);
    		biomesOPlentyStairMapping.put(EnumBlocks.acaciaPlank.meta, Blocks.acaciaStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.cherryPlank.meta, Blocks.cherryStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.darkPlank.meta, Blocks.darkStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.firPlank.meta, Blocks.firStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.holyPlank.meta, Blocks.holyStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.magicPlank.meta, Blocks.magicStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.mangrovePlank.meta, Blocks.mangroveStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.palmPlank.meta, Blocks.palmStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.redwoodPlank.meta, Blocks.redwoodStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.willowPlank.meta, Blocks.willowStairs.orNull());
    		// metadata 10 - Bamboo don't have corresponding stairs
    		biomesOPlentyStairMapping.put(EnumBlocks.pinePlank.meta, Blocks.pineStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.hellBarkPlank.meta, Blocks.hellBarkStairs.orNull());
    		biomesOPlentyStairMapping.put(EnumBlocks.jacarandaPlank.meta, Blocks.jacarandaStairs.orNull());
    	}
    }
    
    /**
     * Map of wood ID -> (Block instance, metadata) for BoP
     */
    private Map<Integer, Pair<Block, Integer>> biomesOPlentyWoodslabMapping = null;
    private void initBOPWoodslabMapping()
    {
    	if( null == biomesOPlentyWoodslabMapping )
    	{
    		/**
    		 * We really would like to use EnumBlocks.acaciaPlanks.meta and so on here, but there's
    		 * no such mapping for the single or double slabs.
    		 */
    		biomesOPlentyWoodslabMapping = new HashMap<Integer, Pair<Block,Integer>>(14);
    		biomesOPlentyWoodslabMapping.put( 0, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 0));
    		biomesOPlentyWoodslabMapping.put( 1, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 1));
    		biomesOPlentyWoodslabMapping.put( 2, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 2));
    		biomesOPlentyWoodslabMapping.put( 3, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 3));
    		biomesOPlentyWoodslabMapping.put( 4, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 4));
    		biomesOPlentyWoodslabMapping.put( 5, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 5));
    		biomesOPlentyWoodslabMapping.put( 6, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 6));
    		biomesOPlentyWoodslabMapping.put( 7, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab1.orNull(), 7));
    		biomesOPlentyWoodslabMapping.put( 8, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab2.orNull(), 0));
    		biomesOPlentyWoodslabMapping.put( 9, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab2.orNull(), 1));
    		// Nothing for Bamboo (10)
    		biomesOPlentyWoodslabMapping.put(11, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab2.orNull(), 2));
    		biomesOPlentyWoodslabMapping.put(12, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab2.orNull(), 3));
    		biomesOPlentyWoodslabMapping.put(13, ImmutablePair.<Block,Integer>of(Blocks.woodenSingleSlab2.orNull(), 4));
    	}
    }
    
    public BiomeSpecificBlock getSpecificBlock(Block block, ForgeDirection dir, BiomeGenBase biome)
    {
    	return BiomeSpecificBlock.query(block, Dir.getMeta(block, dir, coordBaseMode), biome);
    }
    
    // Mod-specific area
    
    /**
     * Stairs to walk on
     */
    public BiomeSpecificBlock getStairs(Block wood, ForgeDirection dir, BiomeGenBase biome)
    {
    	if( Taverns.config.biomesOPlenty )
    	{
    		initBOPStairMapping();
    		// Check which planks we'll get
    		BiomeSpecificBlock planks = BiomeSpecificBlock.query(Block.planks, 0, biome);
			if( planks.block == Blocks.planks.orNull() )
			{
				Block bopStairBlock = biomesOPlentyStairMapping.get(planks.metadata);
				if( null != bopStairBlock )
				{
					return BiomeSpecificBlock.queryVanilla(bopStairBlock, Dir.getMeta(Block.stairsWoodOak, dir, coordBaseMode), null);
				}
			}
    	}
    	// Just use default wood
    	return BiomeSpecificBlock.query(wood, Dir.getMeta(Block.stairsWoodOak, dir, coordBaseMode), biome);
    }
    
    /**
     * Benches to sit on (in vanilla: stairs)
     */
    public BiomeSpecificBlock getBench(Block wood, ForgeDirection dir, BiomeGenBase biome)
    {
    	return getStairs(wood, dir, biome);
    }
    
    public BiomeSpecificBlock getWoodSlab(int plankMeta, ForgeDirection dir, BiomeGenBase biome)
    {
    	if( Taverns.config.biomesOPlenty )
    	{
    		initBOPWoodslabMapping();
    		// Check which planks we'll get
    		BiomeSpecificBlock planks = BiomeSpecificBlock.query(Block.planks, plankMeta, biome);
			if( planks.block == Blocks.planks.orNull() )
			{
				Pair<Block,Integer> bopPlankBlock = biomesOPlentyWoodslabMapping.get(planks.metadata);
				if( null != bopPlankBlock && null != bopPlankBlock.getLeft() )
				{
					return BiomeSpecificBlock.queryVanilla(bopPlankBlock.getLeft(), bopPlankBlock.getRight() + (dir == ForgeDirection.UP ? 8 : 0), null);
				}
			}
    	}
    	// Just use default wood
    	return BiomeSpecificBlock.query(Block.woodSingleSlab, plankMeta + (dir == ForgeDirection.UP ? 8 : 0), biome);
    }
    
    public BiomeSpecificBlock getMossyCobble(BiomeGenBase biome)
    {
    	if( Taverns.config.biomesOPlenty )
    	{
    		// Check what normal cobble turns to
    		BiomeSpecificBlock cobble = BiomeSpecificBlock.query(Block.cobblestone, 0, biome);
    		// Wood -> dead wood
    		if( cobble.block == Blocks.logs1.orNull() || cobble.block == Blocks.logs2.orNull()
    				|| cobble.block == Blocks.logs3.orNull() || cobble.block == Blocks.logs4.orNull() )
    		{
    			return BiomeSpecificBlock.queryVanilla(EnumBlocks.deadLog.block.orNull(), EnumBlocks.deadLog.meta, null);
    		}
    	}
		return BiomeSpecificBlock.query(Block.cobblestoneMossy, 0, biome);
    }
}
