package de.vernideas.mc.taverns.gen;

import gaia.GaiaItem;

import java.util.List;
import java.util.Random;

import biomesoplenty.api.Blocks;
import net.minecraft.block.Block;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.gen.structure.ComponentVillage;
import net.minecraft.world.gen.structure.ComponentVillageStartPiece;
import net.minecraft.world.gen.structure.MapGenStructureIO;
import net.minecraft.world.gen.structure.StructureBoundingBox;
import net.minecraft.world.gen.structure.StructureComponent;
import net.minecraftforge.common.ChestGenHooks;
import net.minecraftforge.common.ForgeDirection;
import cpw.mods.fml.common.ObfuscationReflectionHelper;
import de.vernideas.mc.common.ai.GuardAI;
import de.vernideas.mc.common.gen.BiomeSpecificBlock;
import de.vernideas.mc.common.util.TimePeriod;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;
import de.vernideas.mc.taverns.direction.Dir;

/**
 * Variant on the default tavern
 */
public class ComponentVillageTavern2 extends ComponentVillageBase
{
    public static boolean initialized = false;;
    
    public static void init(Settings config)
    {
    	if( !initialized )
    	{
    		initialized = true;
    		// Make sure the base tavern is initalised too.
    		// We need it for the tavern chest
    		ComponentVillageTavern.init(config);
    		
            MapGenStructureIO.func_143031_a(ComponentVillageTavern2.class, "taverns:ViT2");
    	}
    }
    
    private static final int HEIGHT = 10;
    
    public static Object buildComponent(ComponentVillageStartPiece startPiece, List pieces, Random random,
    		int x, int y, int z, int direction, int type)
    {
    	StructureBoundingBox structBB = StructureBoundingBox.getComponentToAddBoundingBox(x, y, z, 0, 0, 0, 14, HEIGHT, 8, direction);
    	if( canVillageGoDeeper(structBB) )
    	{
            if( null == StructureComponent.findIntersecting(pieces, structBB) )
            {
                    return new ComponentVillageTavern2(startPiece, type, random, structBB, direction);
            }
    	}
    	return null;
    }
    
    private int averageGroundLevel = -1;

    // Because Minecraft likes to call the generation algorithm four times at the same location due to chunk boundaries ...
    private boolean hasMadeChest;
    private int carpetColor;
    private int flower;
    private boolean villagerSpawned = false;

    public ComponentVillageTavern2() {}
    
	public ComponentVillageTavern2(ComponentVillageStartPiece startPiece, int pieces, Random rnd, StructureBoundingBox structBB, int direction)
	{
		super(startPiece, pieces);
        coordBaseMode = direction;
        boundingBox = structBB;
        carpetColor = rnd.nextInt(16);
        flower = rnd.nextInt(16) - 4; // Negative numbers = no flowerpot
    }

	/** writeNBTTag */
    @Override
    protected void func_143012_a(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143012_a(par1NBTTagCompound);
        par1NBTTagCompound.setBoolean("Chest", hasMadeChest);
        par1NBTTagCompound.setInteger("CarpetC", carpetColor);
        par1NBTTagCompound.setInteger("Flower", flower);
        par1NBTTagCompound.setBoolean("VillagerSpawned", villagerSpawned);
    }

	/** readNBTTag */
    @Override
    protected void func_143011_b(NBTTagCompound par1NBTTagCompound)
    {
        super.func_143011_b(par1NBTTagCompound);
        hasMadeChest = par1NBTTagCompound.getBoolean("Chest");
        carpetColor = par1NBTTagCompound.getInteger("CarpetC");
        flower = par1NBTTagCompound.getInteger("Flower");
        villagerSpawned = par1NBTTagCompound.getBoolean("VillagerSpawned");
    }

	@Override
	public boolean addComponentParts(World world, Random rnd, StructureBoundingBox bb)
	{
        if( averageGroundLevel < 0 )
        {
            averageGroundLevel = getAverageGroundLevel(world, bb) + 5;
            if (averageGroundLevel < 0)
            {
                return true;
            }
            boundingBox.offset(0, this.averageGroundLevel - boundingBox.maxY + HEIGHT - 2, 0);
        }

		ComponentVillageStartPiece startPiece = ObfuscationReflectionHelper.getPrivateValue(ComponentVillage.class, this, "startPiece");
		BiomeGenBase biome = (startPiece == null ? null : startPiece.biome);
		
		// Blocks used (for the most part)
		BiomeSpecificBlock planks = BiomeSpecificBlock.query(Block.planks, 0, biome);
		BiomeSpecificBlock cobble = BiomeSpecificBlock.query(Block.cobblestone, 0, biome);
		BiomeSpecificBlock dirt = BiomeSpecificBlock.query(Block.dirt, 0, biome);
		BiomeSpecificBlock gravel = BiomeSpecificBlock.query(Block.gravel, 0, biome);
		BiomeSpecificBlock woodUp = BiomeSpecificBlock.query(Block.wood, 0, biome);
		BiomeSpecificBlock woodWest = getSpecificBlock(Block.wood, ForgeDirection.WEST, biome);
		BiomeSpecificBlock glassPane = BiomeSpecificBlock.query(Block.thinGlass, 0, biome);
		BiomeSpecificBlock fence = BiomeSpecificBlock.query(Block.fence, 0, biome);
		BiomeSpecificBlock carpet = BiomeSpecificBlock.query(Block.carpet, carpetColor, biome);
		
		// Stairs, two variants - roof and actual proper stairs (also for benches)
		BiomeSpecificBlock roofNorth = getSpecificBlock(Block.stairsWoodOak, ForgeDirection.NORTH, biome);
		BiomeSpecificBlock roofSouth = getSpecificBlock(Block.stairsWoodOak, ForgeDirection.SOUTH, biome);
		BiomeSpecificBlock roofWest = getSpecificBlock(Block.stairsWoodOak, ForgeDirection.WEST, biome);
		BiomeSpecificBlock roofEast = getSpecificBlock(Block.stairsWoodOak, ForgeDirection.EAST, biome);
		
		BiomeSpecificBlock stairsEast = getStairs(Block.stairsWoodOak, ForgeDirection.EAST, biome);

		BiomeSpecificBlock benchWest = getBench(Block.stairsWoodOak, ForgeDirection.WEST, biome);
		BiomeSpecificBlock benchEast = getBench(Block.stairsWoodOak, ForgeDirection.EAST, biome);
		BiomeSpecificBlock darkBenchNorth = getBench(Block.stairsWoodSpruce, ForgeDirection.NORTH, biome);
		BiomeSpecificBlock darkBenchEast = getBench(Block.stairsWoodSpruce, ForgeDirection.EAST, biome);
		
        // Clear area for the tavern building
        fill(world, bb, 0,0,0, 14,9,7, BiomeSpecificBlock.air, BiomeSpecificBlock.air);
        for( int xx = 0; xx < 14; ++ xx )
        {
            for( int zz = 0; zz < 8; ++ zz)
            {
                clearCurrentPositionBlocksUpwards(world, xx,0,zz, bb);
                fillDownwards(world, Block.cobblestone, 0, xx,-1,zz, bb);
            }
        }
                
        /*
        if( !hasMadeChest )
        {
        	int chestX = getXWithOffset(9, 2);
        	int chestY = getYWithOffset(5);
        	int chestZ = getZWithOffset(9, 2);
        	if( bb.isVecInside(chestX, chestY, chestZ) )
        	{
        		hasMadeChest = true;
                generateStructureChestContents(world, bb, rnd, 9,5,2,
                		ChestGenHooks.getItems(ComponentVillageTavern.TAVERN_CHEST, rnd),
                		ChestGenHooks.getCount(ComponentVillageTavern.TAVERN_CHEST, rnd));
        	}
        }
        */
        
        /*
        if( !villagerSpawned )
        {
        	EntityVillager villager = spawnVillager(world, bb, 2,1,2);
        	if( null != villager )
        	{
        		villagerSpawned = true;
        		int globalX = getXWithOffset(2, 2);
        		int globalY = getYWithOffset(1);
        		int globalZ = getZWithOffset(2, 2);
        		villager.tasks.addTask(4, new GuardAI(villager, new ChunkCoordinates(globalX, globalY, globalZ),
        				1.0f, 48.0f, new TimePeriod(0.15, 0.75), false));
        	}
        }
        */
        
        return true;
	}
	
	@Override
	protected int getVillagerType(int alreadySpawned)
	{
		return (Taverns.config.barWenchID > 0 ? Taverns.config.barWenchID : super.getVillagerType(alreadySpawned));
	}
}
