package de.vernideas.mc.taverns.trader;

import gaia.GaiaItem;

import java.util.Random;

import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ResourceLocation;
import net.minecraft.village.MerchantRecipe;
import net.minecraft.village.MerchantRecipeList;

import org.millenaire.common.forge.Mill;

import biomesoplenty.api.Items;
import cpw.mods.fml.common.registry.VillagerRegistry;
import cpw.mods.fml.common.registry.VillagerRegistry.IVillageTradeHandler;
import de.vernideas.mc.common.ai.GuardAI;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;

public class VillageHandlerBarWench extends VillageHandlerBase implements IVillageTradeHandler
{
	public static VillageHandlerBarWench villageHandler = null;
	
	public static void init(Settings config)
	{
		if( null == villageHandler )
		{
			villageHandler = new VillageHandlerBarWench();
			VillagerRegistry.instance().registerVillagerId(config.barWenchID);
			VillagerRegistry.instance().registerVillageTradeHandler(config.barWenchID, villageHandler);
			VillagerRegistry.instance().registerVillagerSkin(config.barWenchID,
					new ResourceLocation(Taverns.modID, "textures/entity/barwench.png"));
		}
	}
	
	@Override
	public void manipulateTradesForVillager(EntityVillager villager, MerchantRecipeList recipeList, Random random)
	{
		ItemStack emeraldShard = null;
		if( Taverns.config.grimoireOfGaia2 )
		{
			emeraldShard = new ItemStack(GaiaItem.ShardEmerald, 1);
		}
		
		// Awkward potions aren't stackable, so we have two single-item stacks of them
        recipeList.add(new MerchantRecipe(
                new ItemStack(Item.potion, 1, 16), new ItemStack(Item.potion, 1, 16),
                new ItemStack(Item.emerald, 1)));
        // Night vision
        addRandomTrade(recipeList, emerald, new ItemStack(Item.potion, 2, 33), 0.5, random);
        addRandomTrade(recipeList, emerald, randomItem(Item.bakedPotato, 28, 40, random), 0.9, random);
        addRandomTrade(recipeList, emerald, randomItem(Item.magmaCream, 4, 8, random), 0.3, random);

        // Quest type trades
        addRandomTrade(recipeList, new ItemStack(Item.bone, 12), emerald, 0.5, random);
        addRandomTrade(recipeList, new ItemStack(Item.glowstone, 12), emerald, 0.5, random);
        addRandomTrade(recipeList, new ItemStack(Item.blazePowder, 12), emerald, 0.5, random);
        
        if( Taverns.config.grimoireOfGaia2 )
        {
        	// Use GoG2's emerald shard (=1/4th of an emerald) for some trades too.
        	addRandomTrade(recipeList, new ItemStack(Item.potion, 1, 16), emeraldShard, 0.7, random);
            addRandomTrade(recipeList, new ItemStack(GaiaItem.ShardEmerald, 2), new ItemStack(Item.potion, 1, 33), 0.5, random); // night vision
            addRandomTrade(recipeList, emeraldShard, randomItem(Item.bakedPotato, 7, 10, random), 0.9, random);
            addRandomTrade(recipeList, new ItemStack(Item.bone, 3), emeraldShard, 0.5, random);
            addRandomTrade(recipeList, new ItemStack(Item.glowstone, 3), emeraldShard, 0.5, random);
            addRandomTrade(recipeList, new ItemStack(Item.blazePowder, 3), emeraldShard, 0.5, random);
        }
        
        if( Taverns.config.millenaire )
        {
        	// Lots of drinks
        	addRandomTrade(recipeList, emerald, new ItemStack(Mill.calva, 2), 0.3, random);
        	addRandomTrade(recipeList, emerald, new ItemStack(Mill.cider, 4), 0.5, random);
        	addRandomTrade(recipeList, emerald, randomItem(Mill.wineBasic, 4, 8, random), 0.8, random);
        	addRandomTrade(recipeList, emerald, new ItemStack(Mill.wineFancy, 1), 0.2, random);
        	addRandomTrade(recipeList, emerald, new ItemStack(Mill.cacauhaa, 1), 0.2, random);
        	addRandomTrade(recipeList, emerald, new ItemStack(Mill.sake, 2), 0.3, random);

           	// Buy stuff ...
           	addRandomTrade(recipeList, randomItem(Mill.ciderapple, 10, 14, random), emerald, 0.5, random);
           	addRandomTrade(recipeList, randomItem(Mill.grapes, 6, 10, random), emerald, 0.5, random);
           	
           	// GoG2 variants on some trades
           	if( Taverns.config.grimoireOfGaia2 )
           	{
           		addRandomTrade(recipeList, new ItemStack(GaiaItem.ShardEmerald, 2), new ItemStack(Mill.calva, 1), 0.3, random);
            	addRandomTrade(recipeList, emeraldShard, new ItemStack(Mill.cider, 1), 0.5, random);
            	addRandomTrade(recipeList, emeraldShard, randomItem(Mill.wineBasic, 1, 2, random), 0.8, random);
               	addRandomTrade(recipeList, new ItemStack(GaiaItem.ShardEmerald, 2), new ItemStack(Mill.sake, 1), 0.3, random);

               	addRandomTrade(recipeList, randomItem(Mill.ciderapple, 2, 4, random), emeraldShard, 0.5, random);
               	addRandomTrade(recipeList, new ItemStack(Mill.grapes, 2), emeraldShard, 0.5, random);
           	}
        }

        if( Taverns.config.biomesOPlenty )
        {
        	if( Items.food.isPresent() )
        	{
	        	// Trades don't differentiate between same itemID but different damageValues, so for now
	        	// (until implementing own EntityVillager) it's one or the other.
	        	if( random.nextBoolean() || MinecraftServer.getServer().getMinecraftVersion().equals("1.6.2") )
	        	{
	        		// Berries
	                recipeList.add(new MerchantRecipe(randomItem(Items.food.get(), 28, 36, 0, random), emerald));
	        	}
	        	else
	        	{
	        		// Persimmons
	                recipeList.add(new MerchantRecipe(randomItem(Items.food.get(), 20, 28, 8, random), emerald));
	        	}
	        	
	           	// GoG2 variants on some trades
	           	if( Taverns.config.grimoireOfGaia2 )
	           	{
	            	if( random.nextBoolean() || MinecraftServer.getServer().getMinecraftVersion().equals("1.6.2") )
	            	{
	                    recipeList.add(new MerchantRecipe(randomItem(Items.food.get(), 7, 9, 0, random), emeraldShard));
	            	}
	            	else
	            	{
	                    recipeList.add(new MerchantRecipe(randomItem(Items.food.get(), 5, 7, 8, random), emeraldShard));
	            	}
	           	}
        	}
        }
	}
}
