package de.vernideas.mc.taverns.trader;

import gaia.GaiaItem;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraft.village.MerchantRecipe;
import net.minecraft.village.MerchantRecipeList;
import cpw.mods.fml.common.registry.VillagerRegistry;
import cpw.mods.fml.common.registry.VillagerRegistry.IVillageTradeHandler;
import de.vernideas.mc.taverns.Settings;
import de.vernideas.mc.taverns.Taverns;

public class VillageHandlerShepherdess extends VillageHandlerBase implements IVillageTradeHandler
{
	public static VillageHandlerShepherdess villageHandler = null;
	
	public static void init(Settings config)
	{
		if( null == villageHandler )
		{
			villageHandler = new VillageHandlerShepherdess();
			VillagerRegistry.instance().registerVillagerId(config.shepherdessID);
			VillagerRegistry.instance().registerVillageTradeHandler(config.shepherdessID, villageHandler);
			VillagerRegistry.instance().registerVillagerSkin(config.shepherdessID,
					new ResourceLocation(Taverns.modID, "textures/entity/village_shepherdess.png"));
		}
	}

	@Override
	public void manipulateTradesForVillager(EntityVillager villager, MerchantRecipeList recipeList, Random random)
	{
		ItemStack emeraldShard = null;
		if( Taverns.config.grimoireOfGaia2 )
		{
			emeraldShard = new ItemStack(GaiaItem.ShardEmerald, 1);
		}

		// Wool to string
		recipeList.add(new MerchantRecipe(new ItemStack(Block.cloth, 1), new ItemStack(Item.silk, 4)));
		
		// Merchant recipes treat items with different damageValue but same itemID as the same, so
		// we can only use one of those trades, sadly ...
		switch( random.nextInt(5) )
		{
			case 0:
				// White
				recipeList.add(new MerchantRecipe(emerald, randomItem(Block.cloth, 14, 18, 0, random)));
				break;
			case 1:
				// Light grey
				recipeList.add(new MerchantRecipe(emerald, randomItem(Block.cloth, 10, 14, 8, random)));
				break;
			case 2:
				// Dark grey
				recipeList.add(new MerchantRecipe(emerald, randomItem(Block.cloth, 10, 14, 7, random)));
				break;
			case 3:
				// Black
				recipeList.add(new MerchantRecipe(emerald, randomItem(Block.cloth, 6, 12, 15, random)));
				break;
			case 4:
				// Any random
				recipeList.add(new MerchantRecipe(emerald, randomItem(Block.cloth, 6, 12, random.nextInt(16), random)));
				break;
			default:
				// This shouldn't happen.
				break;
		}
        
        if( Taverns.config.grimoireOfGaia2 )
        {
        	// Use GoG2's emerald shard (=1/4th of an emerald) for some trades too.
            recipeList.add(new MerchantRecipe(emeraldShard, new ItemStack(Item.silk, 6)));
    		switch( random.nextInt(5) )
    		{
    			case 0:
    				// White
    				recipeList.add(new MerchantRecipe(emeraldShard, randomItem(Block.cloth, 3, 5, 0, random)));
    				break;
    			case 1:
    				// Light grey
    				recipeList.add(new MerchantRecipe(emeraldShard, randomItem(Block.cloth, 2, 3, 8, random)));
    				break;
    			case 2:
    				// Dark grey
    				recipeList.add(new MerchantRecipe(emeraldShard, randomItem(Block.cloth, 2, 3, 7, random)));
    				break;
    			case 3:
    				// Black
    				recipeList.add(new MerchantRecipe(emeraldShard, new ItemStack(Block.cloth, 2, 15)));
    				break;
    			case 4:
    				// Any random
    				recipeList.add(new MerchantRecipe(emeraldShard, new ItemStack(Block.cloth, 2, random.nextInt(16))));
    				break;
    			default:
    				// This shouldn't happen.
    				break;
    		}
    		addRandomTrade(recipeList, new ItemStack(Item.shears, 1, 0), emeraldShard, 0.5, random);
        }
	}

}
